// MM1ALG.CPP

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "mm1alg.h"

////////////////////////////////////////////////////////////////////////////////////////////////////

// if we are doing geometry optimization or molecular dynamics for periodic models,
// we should also check (at suitable intervals) that molecules are not drifting away
// from our simulation box. there is CheckLocations()-function in the engine class
// to do this trick, just remember to call it...

////////////////////////////////////////////////////////////////////////////////////////////////////

mm1_geomopt::mm1_geomopt(mm1_eng * p1, i32s p2, f64 p3) : conjugate_gradient(p2, p3)
{
	eng = p1;
	
	for (i32u n1 = 0;n1 < eng->GetModel()->atom_list.size();n1++)
	{
		for (i32u n2 = 0;n2 < 3;n2++)
		{
			AddVar(& eng->crd[n1][n2], & eng->d1[n1][n2]);
		}
	}
}

mm1_geomopt::~mm1_geomopt(void)
{
}

f64 mm1_geomopt::GetValue(void)
{
	eng->Compute(1);
	return eng->energy;
}

f64 mm1_geomopt::GetGradient(void)
{
	eng->Compute(1);
	return eng->energy;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

// void mm1_moldyn::StopTranslation(void), void mm1_moldyn::StopRotation(void) ???

// so far very long simulations have not been tested, so possile translation/rotation
// problems are not solved... current solution is to start simulations from 0 K ->
// no translation/rotation in the initial state -> conservation of linear and angular
// momentum -> no need to worry at all... but for how long it will work???

mm1_moldyn::mm1_moldyn(mm1_eng * p1, f64 p2, f64 p3)
{
	eng = p1;
	
	vel = new f64[eng->GetModel()->atom_list.size() * 3];		// [1.0E+3 m/s]
	acc = new f64[eng->GetModel()->atom_list.size() * 3];		// [1.0E+12 m/s^2]
	
	mass = new f64[eng->GetModel()->atom_list.size()];
	
	iter_mm1al it1 = eng->GetModel()->atom_list.begin(); i32s n1 = 0;
	while (it1 != eng->GetModel()->atom_list.end())
	{
		mass[n1] = (* it1).el.GetAtomicMass();
		mass[n1] *= 1.6605402e-27 * 6.0221367e+23;
		
		for (i32s n2 = 0;n2 < 3;n2++)
		{
			vel[n1 * 3 + n2] = 0.0;
			acc[n1 * 3 + n2] = 0.0;
		}
		
		it1++; n1++;
	}
	
	temp = p2; step1 = p3;
	step2 = step1 * step1;		// [1.0E-15 s]
}

mm1_moldyn::~mm1_moldyn(void)
{
	delete[] vel;
	delete[] acc;
	
	delete[] mass;
}

// Allen MP, Tildesley DJ : "Computer Simulation of Liquids", Clarendon Press, 1987

// this is a "velocity-Verlet"-type integrator...

void mm1_moldyn::TakeMDStep(void)
{
	for (i32u n1 = 0;n1 < eng->GetModel()->atom_list.size();n1++)
	{
		for (i32u n2 = 0;n2 < 3;n2++)
		{
			f64 tmp1 = step1 * vel[n1 * 3 + n2] * 1.0e-3;
			f64 tmp2 = step2 * acc[n1 * 3 + n2] * 0.5e-9;
			eng->crd[n1][n2] += tmp1 + tmp2;
			
			vel[n1 * 3 + n2] += step1 * acc[n1 * 3 + n2] * 0.5e-6;
		}
	}
	
	eng->Compute(1);
	pot = eng->energy;
	
	for (i32u n1 = 0;n1 < eng->GetModel()->atom_list.size();n1++)
	{
		for (i32u n2 = 0;n2 < 3;n2++)
		{
			acc[n1 * 3 + n2] = -eng->d1[n1][n2] / mass[n1];
			vel[n1 * 3 + n2] += step1 * acc[n1 * 3 + n2] * 0.5e-6;
		}
	}
	
	kin = KineticEnergy();
}

f64 mm1_moldyn::KineticEnergy(void)
{
	f64 energy = 0.0;
	for (i32u n1 = 0;n1 < eng->GetModel()->atom_list.size();n1++)
	{
		f64 sum = 0.0;
		for (i32s n2 = 0;n2 < 3;n2++)
		{
			f64 tmp = vel[n1 * 3 + n2];
			sum += tmp * tmp;
		}
		
		energy += 500.0 * mass[n1] * sum;
	}
	
	return energy;
}

f64 mm1_moldyn::ConvTempEKin(f64 p1)
{
	return (3.0 / 2.0) * eng->GetModel()->atom_list.size() * 8.314510 * p1 / 1000.0;
}

f64 mm1_moldyn::ConvEKinTemp(f64 p1)
{
	return (2.0 / 3.0) * p1 * 1000.0 / (eng->GetModel()->atom_list.size() * 8.314510);
}

void mm1_moldyn::SetEKin(f64 p1)
{
	f64 tmp1 = p1 / KineticEnergy();
	f64 tmp2 = (tmp1 < 0.0 ? 0.0 : sqrt(tmp1));
	
	i32u tmp3 = eng->GetModel()->atom_list.size() * 3;
	for (i32u n1 = 0;n1 < tmp3;n1++) vel[n1] *= tmp2;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

// eof
