// OBJECTS.H : base classes for different 3D-objects.

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "config.h"	// this is target-dependent...

#ifndef OBJECTS_H
#define OBJECTS_H

#ifdef ENABLE_GUI

struct obj_loc_data;

class dummy_object;
class smart_object;

class object_location;

class ol_static;
class ol_dynamic_l1;

////////////////////////////////////////////////////////////////////////////////////////////////////

class camera;		// viewing.h
class window;		// window.h

#include "v3d.h"
#include "define.h"

#include <vector>

////////////////////////////////////////////////////////////////////////////////////////////////////

void SetModelView(const obj_loc_data *);

// directions are similar to those in OpenGL:
// "zdir" -> positive direction is to the forward.
// "ydir" -> positive direction is to the upward.
// "xdir" -> positive direction is to the left.

// all vectors are supposed to be unit vectors !!!
// only lights will use the fourth component !!!

struct obj_loc_data
{
	fGL crd[4];
	v3d<fGL> zdir;
	v3d<fGL> ydir;
	
	i32s lock_count;
};

////////////////////////////////////////////////////////////////////////////////////////////////////

// dummy_object is a base class for many internally used objects (like cameras, lights, clippers
// and transformers), and contains interfaces for transformations, simple modifications and rendering.

// some objects (like ribbons) are fixed so not all objects necessarily have object_location...

class dummy_object
{
	protected:
	
	obj_loc_data * GetLocDataRW(void) const;
	
	object_location * ol;
	
	public:
	
	dummy_object(bool);
	dummy_object(const object_location &);
	virtual ~dummy_object(void);
	
	void SetModelView(void) const;
	const obj_loc_data * GetLocData(void) const;
	
	virtual const char * GetObjectName(void) = 0;
	
	virtual void OrbitObject(const fGL *, const camera &);
	virtual void RotateObject(const fGL *, const camera &);
	
	virtual void TranslateObject(const fGL *, const obj_loc_data *);

	virtual bool BeginTransformation(void) = 0;
	virtual bool EndTransformation(void) = 0;
	
	virtual void Render(void) = 0;
};

////////////////////////////////////////////////////////////////////////////////////////////////////

struct smart_client
{
	window * client;
	void * custom_data;
};

// UNDER CONSTRUCTION!!!!! UNDER CONSTRUCTION!!!!! UNDER CONSTRUCTION!!!!!
// UNDER CONSTRUCTION!!!!! UNDER CONSTRUCTION!!!!! UNDER CONSTRUCTION!!!!!
// UNDER CONSTRUCTION!!!!! UNDER CONSTRUCTION!!!!! UNDER CONSTRUCTION!!!!!

// smart_objects are used to implement complex visualization objects (like colored 3D-planes and
// surfaces, and ribbon models of peptides/proteins).

// smart_object can respond to connectivity/geometry changes and can have window-dependent
// customized properties. clippers can be used to clip smart_objects (AND ONLY THOSE?????).

// "new connectivity" means that some atoms/bonds have been added/removed.
// "new geometry" means that some atom coordinates have been modified.

class smart_object : public dummy_object
{
	public:
	
	bool transparent; fGL alpha;
	vector<smart_client> client_vector;
	
	public:
	
	smart_object(void);
	smart_object(const object_location &);
	virtual ~smart_object(void);
	
//	virtual bool AddClient(window *, void *) = 0;
//	virtual bool RemoveClient(window *) = 0;
	
//	virtual void NewConnectivity(void) = 0;
//	virtual void NewGeometry(void) = 0;
};

////////////////////////////////////////////////////////////////////////////////////////////////////

// object_location is an abstract base class for all objects(???). it contains information
// about the position and orientation of the object and is also able to write that information
// into the OpenGL modelview matrix.

// idea is to create derived classes that can have different (also dynamic) ways to determine
// the location...

class object_location
{
	protected:
	
	friend obj_loc_data * dummy_object::GetLocDataRW(void) const;
	
	obj_loc_data * data;
	
	public:
	
	object_location(void);
	object_location(const object_location &);
	virtual ~object_location(void);
	
	void SetModelView(void) const;
	const obj_loc_data * GetLocData(void) const;
	
	virtual void UpdateLocation(void) const = 0;
	virtual object_location * MakeCopy(void) const = 0;
};

////////////////////////////////////////////////////////////////////////////////////////////////////

// here we have some derived location classes...

class ol_static : public object_location
{
	protected:
	
// this is just an interface definition -> there is no relevant common data?!?!?
// this is just an interface definition -> there is no relevant common data?!?!?
// this is just an interface definition -> there is no relevant common data?!?!?

	public:
	
	ol_static(void);
	~ol_static(void);
	
	void UpdateLocation(void) const;
	object_location * MakeCopy(void) const;
};

class ol_dynamic_l1 : public object_location
{
	private:
	
// bring the points and weights here?!?!?!?!?
// bring the points and weights here?!?!?!?!?
// bring the points and weights here?!?!?!?!?

	public:
	
	ol_dynamic_l1(void);
	~ol_dynamic_l1(void);
	
	void UpdateLocation(void) const;
	object_location * MakeCopy(void) const;
};

////////////////////////////////////////////////////////////////////////////////////////////////////

inline void dummy_object::SetModelView(void) const
{
	if (ol != NULL) ol->SetModelView();
}

inline const obj_loc_data * dummy_object::GetLocData(void) const
{
	return ol != NULL ? ol->GetLocData() : NULL;
}

inline obj_loc_data * dummy_object::GetLocDataRW(void) const
{
	return ol != NULL ? ol->data : NULL;
}

inline const obj_loc_data * object_location::GetLocData(void) const
{
	return data;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// ENABLE_GUI
#endif	// OBJECTS_H

// eof
