// WINDOW.H : a view window base class, easy to use on different platforms...

// Copyright (C) 1998 Tommi Hassinen, Jarno Huuskonen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "config.h"	// this is target-dependent...

#ifndef WINDOW_H
#define WINDOW_H

#ifdef ENABLE_GUI

class window;

class view_window;
class plot_window;

class plot2d_window;
class plot3d_window;

class window_factory;

////////////////////////////////////////////////////////////////////////////////////////////////////

class camera;		// viewing.h

class color_mode;	// mdlbase.h
class bs_gui;		// guibase.h

#include "mtools.h"

////////////////////////////////////////////////////////////////////////////////////////////////////

// "window" is a base class for all windows. all operating system/windowing environment-
// dependent things are wrapped in ctor/dtor or in a set of pure virtual functions.
// some mouse tool-related and other utility stuff is also here...

class window
{
	public:
	
	bs_gui * gui;
	
	i32s size[2];		// size of the window, in pixels.
	fGL range[2];		// dimensions of the view, in nanometers.
	
	public:
	
	// static data about mouse/keyboard state...
	
	static mouse_tool::state state;
	static mouse_tool::button button;
	
	static bool shift_down;
	static bool ctrl_down;
	
	// all available mouse tools are here...
	
	static draw_tool tool_draw;
	static erase_tool tool_erase;
	
	static select_tool tool_select;
	static zoom_tool tool_zoom;
	
	static translate_xy_tool tool_translate_xy;
	static translate_z_tool tool_translate_z;
	
	static orbit_xy_tool tool_orbit_xy;
	static orbit_z_tool tool_orbit_z;
	
	static rotate_xy_tool tool_rotate_xy;
	static rotate_z_tool tool_rotate_z;
	
	// pointer to the currently selected mouse tool.
	
	static mouse_tool * current_tool;
	
	public:
	
	window(bs_gui *);
	virtual ~window(void);
	
	void SetSize(i32s, i32s);
	
	void WriteBitmapString(void *, const char *);
	i32s GetBitmapStringWidth(void *, const char *);
	
	virtual void CreateWindow(void) = 0;
	virtual void SetTitle(const char *) = 0;
	virtual void SetCurrent(void) = 0;
	virtual void Update(bool) = 0;
	
	virtual void Render(void) = 0;
};

////////////////////////////////////////////////////////////////////////////////////////////////////

class view_window : virtual public window
{
	public:
	
	camera * cam;
	
	i32s render; i32s label;
	color_mode * colormode;
	
	bool enable_fog;
	bool accumulate;
	
	// some static flags...
	
	static bool quick_update;
	static bool draw_info;
	
	public:
	
	view_window(bs_gui *, camera *);
	virtual ~view_window(void);
	
	void GetCRD(i32s *, fGL *);
	
	void Render(void);	// virtual
};

////////////////////////////////////////////////////////////////////////////////////////////////////


class plot_window : virtual public window
{
	public:
	
	plot_window(bs_gui *);
	virtual ~plot_window(void);
};

////////////////////////////////////////////////////////////////////////////////////////////////////

class plot2d_window : public plot_window
{
	public:
	
	plot2d_window(bs_gui *);
	virtual ~plot2d_window(void);
};

////////////////////////////////////////////////////////////////////////////////////////////////////

class plot3d_window : public plot_window
{
	public:
	
	plot3d_window(bs_gui *);
	virtual ~plot3d_window(void);
};

////////////////////////////////////////////////////////////////////////////////////////////////////

// "window_factory" is a platform-independent way to create viewing windows. for each different
// platform (just like a "window") a "window_factory" should be made which produces a new "window"
// for this particular platform...

class window_factory
{
	protected:
	
	public:
	
	window_factory(void) { };
	virtual ~window_factory(void) { };
	
	virtual view_window * ProduceView(bs_gui *, camera *) = 0;
};

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// ENABLE_GUI
#endif	// WINDOW_H

// eof
